unit UFormMain;

{$mode Delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, SynEdit, Forms, Controls, Graphics, Dialogs,
  Menus, ActnList, ComCtrls, ExtCtrls, UTarget, Registry, URegistry, Process,
  LazFileUtils;

type

  { TFormMain }

  TFormMain = class(TForm)
    AViewLog: TAction;
    AProgramShowExecutionPoint: TAction;
    AShowSourcePosition: TAction;
    AShowTargetPosition: TAction;
    AProgramCompile: TAction;
    AAbout: TAction;
    ABreakpointToggle: TAction;
    AViewCompilers: TAction;
    AOptions: TAction;
    AProgramStepOut: TAction;
    AProgramRunToCursor: TAction;
    AProgramStepOver: TAction;
    AProgramStepInto: TAction;
    AProgramPause: TAction;
    AProgramStop: TAction;
    AHelp: TAction;
    AProgramRun: TAction;
    AProjectNew: TAction;
    AExit: TAction;
    AProjectSaveAs: TAction;
    AProjectClose: TAction;
    AProjectSave: TAction;
    AProjectOpen: TAction;
    ActionList1: TActionList;
    ImageListMain: TImageList;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem15: TMenuItem;
    MenuItem16: TMenuItem;
    MenuItem17: TMenuItem;
    MenuItem18: TMenuItem;
    MenuItem19: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem22: TMenuItem;
    MenuItem24: TMenuItem;
    MenuItem25: TMenuItem;
    MenuItem27: TMenuItem;
    MenuItem28: TMenuItem;
    MenuItem30: TMenuItem;
    MenuItem31: TMenuItem;
    MenuItem32: TMenuItem;
    MenuItem33: TMenuItem;
    MenuItem34: TMenuItem;
    MenuItem35: TMenuItem;
    MenuItemOpenRecent: TMenuItem;
    MenuItemTarget: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    PageControlMain: TPageControl;
    PageControlRight: TPageControl;
    PageControlBottom: TPageControl;
    PanelLeft: TPanel;
    PanelMemory: TPanel;
    PanelCPU: TPanel;
    PanelOutput: TPanel;
    PanelInput: TPanel;
    PopupMenuTargets: TPopupMenu;
    PopupMenuRecentFiles: TPopupMenu;
    SaveDialog1: TSaveDialog;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    Splitter3: TSplitter;
    Splitter4: TSplitter;
    Splitter5: TSplitter;
    StatusBarMain: TStatusBar;
    TabSheetMessages: TTabSheet;
    TabSheetDebug: TTabSheet;
    TabSheetSource: TTabSheet;
    TabSheetTarget: TTabSheet;
    TimerRefresh: TTimer;
    ToolBarMain: TToolBar;
    ToolButton1: TToolButton;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    ToolButton14: TToolButton;
    ToolButton15: TToolButton;
    ToolButtonTarget: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    procedure AAboutExecute(Sender: TObject);
    procedure ABreakpointToggleExecute(Sender: TObject);
    procedure AExitExecute(Sender: TObject);
    procedure AOptionsExecute(Sender: TObject);
    procedure AProgramCompileExecute(Sender: TObject);
    procedure AProgramPauseExecute(Sender: TObject);
    procedure AProgramRunToCursorExecute(Sender: TObject);
    procedure AProgramRunExecute(Sender: TObject);
    procedure AProgramShowExecutionPointExecute(Sender: TObject);
    procedure AProgramStepIntoExecute(Sender: TObject);
    procedure AProgramStepOutExecute(Sender: TObject);
    procedure AProgramStepOverExecute(Sender: TObject);
    procedure AProgramStopExecute(Sender: TObject);
    procedure AProjectCloseExecute(Sender: TObject);
    procedure AProjectNewExecute(Sender: TObject);
    procedure AProjectOpenExecute(Sender: TObject);
    procedure AProjectSaveAsExecute(Sender: TObject);
    procedure AProjectSaveExecute(Sender: TObject);
    procedure AShowSourcePositionExecute(Sender: TObject);
    procedure AShowTargetPositionExecute(Sender: TObject);
    procedure AViewCompilersExecute(Sender: TObject);
    procedure AViewInterpretterExecute(Sender: TObject);
    procedure AViewLogExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TimerRefreshTimer(Sender: TObject);
  private
    procedure AProjectOpenRecentExecute(Sender: TObject);
    procedure MenuItemTargetClick(Sender: TObject);
    procedure ProjectOpen(FileName: string);
    procedure UpdateToolBarHints;
    procedure DockInit;
    procedure LastOpenedListChange(Sender: TObject);
  public
    procedure MessagesChanged(Sender: TObject);
    procedure TargetLogExecute(Lines: TStrings);
    procedure TargetStateChanged(Sender: TObject);
    procedure LoadFromRegistry(Root: HKEY; Key: string);
    procedure SaveToRegistry(Root: HKEY; Key: string);
    procedure UpdateInterface;
    procedure UpdateStatusBar;
    procedure UpdateTargetList;
  end;

var
  FormMain: TFormMain;


implementation

{$R *.lfm}

uses
  UCore, UFormCPU, UFormOptions, UFormMessages, UFormTargets, UFormOutput, UFormInput,
  UFormMemory, UFormSourceCode, UFormTargetCode, UTargetInterpretter, UFormLog,
  UBFTarget, UFormAbout;

resourcestring
  SProgramStarted = 'Program started';
  SProgramStopped = 'Program stopped';
  SNewProject = 'New project';
  SCompileStart = 'Compiling...';
  SDone = 'Done';
  SChanged = 'Changed';
  SUnsavedChanges = 'Unsaved changes';
  SDoYouWantToSaveProject = 'There are unsaved changes in project. Do you want to save them?';
  SCloseProjectFirst = 'Project need to be closed before it can be openned';

{ TFormMain }

procedure TFormMain.FormShow(Sender: TObject);
begin
  // TODO: Transparent image scaling not working properly under linux Gtk2
  // Also screen DPI is not correctly detected under linux Gtk2
  {$IFDEF WINDOWS}
  with Core.ScaleDPI do begin
    ApplyToAll(DesignDPI);
    ScaleImageList(ImageListMain, DesignDPI);
  end;
  {$ENDIF}

  Core.CoolTranslator1.Translate;
  LoadFromRegistry(HKEY(Core.ApplicationInfo.RegistryRoot), Core.ApplicationInfo.RegistryKey);
  PageControlMain.TabIndex := 0;
  if Core.OpenProjectOnStart and (Core.LastOpenedList.Items.Count > 0) then
    ProjectOpen(Core.LastOpenedList.Items[0])
    else AProjectNew.Execute;
  UpdateInterface;
  UpdateToolBarHints;
  UpdateTargetList;
  DockInit;
  Core.PersistentForm1.Load(Self, wsMaximized);
end;

procedure TFormMain.TimerRefreshTimer(Sender: TObject);
begin
  FormCPU.Reload;
  FormOutput.Reload;
  FormMemory.Reload;
end;

procedure TFormMain.TargetLogExecute(Lines: TStrings);
begin
  FormLog.Memo1.Lines.AddStrings(Lines);
end;

procedure TFormMain.TargetStateChanged(Sender: TObject);
begin
  UpdateInterface;
  if Core.CurrentTarget.State = rsPaused then
    AProgramShowExecutionPoint.Execute;
  if Core.CurrentTarget.State = rsStopped then
  Core.CurrentTarget.Messages.AddMessage(SProgramStopped);
end;

procedure TFormMain.MessagesChanged(Sender: TObject);
begin
  FormMessages.Reload;
end;

procedure TFormMain.MenuItemTargetClick(Sender: TObject);
var
  I: Integer;
begin
  if Sender is TMenuItem then
  with TMenuItem(Sender) do begin
    Core.CurrentTarget := TTarget(Core.Targets[MenuIndex]);
    Checked := True;
    for I := 0 to Parent.Count - 1 do
      if Parent.Items[I].MenuIndex <> MenuIndex then Parent.Items[I].Checked := False;
  end;
end;

procedure TFormMain.ProjectOpen(FileName: string);
begin
  if Core.Project.Openned then
    raise Exception.Create(SCloseProjectFirst);
  FormSourceCode.MemoSource.Lines.LoadFromFile(FileName);
  Core.LastOpenedList.AddItem(FileName);
  Core.Project.FileName := FileName;
  Core.Project.Openned := True;
  Core.CurrentTarget.Compiled := False;
  UpdateInterface;
end;

procedure TFormMain.UpdateToolBarHints;
var
  I: Integer;
begin
  for I := 0 to ToolBarMain.ControlCount - 1 do
    ToolBarMain.Controls[I].Hint := ToolBarMain.Controls[I].Caption;
end;

procedure TFormMain.DockInit;
begin
  FormSourceCode.ManualDock(TabSheetSource, nil, alClient);
  FormSourceCode.Align := alClient;
  FormSourceCode.Show;
  FormTargetCode.ManualDock(TabSheetTarget, nil, alClient);
  FormTargetCode.Align := alClient;
  FormTargetCode.Show;
  FormCPU.ManualDock(PanelCPU, nil, alClient);
  FormCPU.Align := alClient;
  FormCPU.Show;
  FormInput.ManualDock(PanelInput, nil, alClient);
  FormInput.Align := alClient;
  FormInput.Show;
  FormOutput.ManualDock(PanelOutput, nil, alClient);
  FormOutput.Align := alClient;
  FormOutput.Show;
  FormMemory.ManualDock(PanelMemory, nil, alClient);
  FormMemory.Align := alClient;
  FormMemory.Show;
  FormMessages.ManualDock(TabSheetMessages, nil, alClient);
  FormMessages.Align := alClient;
  FormMessages.Show;
  PageControlRight.Width := FormMain.Width div 3;
  PageControlBottom.Height := FormMain.Height div 5;
end;

procedure TFormMain.LastOpenedListChange(Sender: TObject);
begin
  Core.LastOpenedList.LoadToMenuItem(MenuItemOpenRecent, AProjectOpenRecentExecute);
  Core.LastOpenedList.LoadToMenuItem(PopupMenuRecentFiles.Items, AProjectOpenRecentExecute);
end;

procedure TFormMain.LoadFromRegistry(Root: HKEY; Key: string);
var
  TargetName: string;
begin
  with TRegistryEx.Create do
  try
    RootKey := Root;
    OpenKey(Key, True);
    TargetName := ReadStringWithDefault('TargetName', 'Interpretter');
    Core.CurrentTarget := Core.Targets.FindByName(TargetName);
    if not Assigned(Core.CurrentTarget) then
      Core.CurrentTarget := TTarget(Core.Targets[0]);
  finally
    Free;
  end;
end;

procedure TFormMain.SaveToRegistry(Root: HKEY; Key: string);
begin
  with TRegistryEx.Create do
  try
    RootKey := Root;
    OpenKey(Key, True);
    WriteString('TargetName', Core.CurrentTarget.Name);
  finally
    Free;
  end;
end;

procedure TFormMain.UpdateInterface;
var
  Title: string;
begin
  Title := '';
  if Core.Project.FileName <> '' then Title := Core.Project.FileName;
  if Core.Project.Modified then Title := Title + ' (' + SChanged + ')';
  Title:= Title + ' - ' + Core.ApplicationInfo.AppName;
  Caption := Title;
  AProjectSave.Enabled := Core.Project.Modified;
  FormSourceCode.MemoSource.Enabled := Core.Project.Openned;
  AProjectClose.Enabled := Core.Project.Openned;
  AProjectSaveAs.Enabled := Core.Project.Openned;
  AProgramRun.Enabled := (tcRun in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned and (Core.CurrentTarget.State = rsStopped) or (Core.CurrentTarget.State = rsPaused);
  AProgramPause.Enabled := (tcPause in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned and (Core.CurrentTarget.State = rsRunning);
  AProgramStop.Enabled := (tcStop in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned and (Core.CurrentTarget.State <> rsStopped);
  AProgramCompile.Enabled := (tcCompile in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned;
  AProgramStepInto.Enabled := (tcStepInto in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned and ((Core.CurrentTarget.State = rsPaused) or (Core.CurrentTarget.State = rsStopped));
  AProgramStepOut.Enabled := (tcStepOut in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned and (Core.CurrentTarget.State = rsPaused);
  AProgramRunToCursor.Enabled := (tcRunToCursor in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned and ((Core.CurrentTarget.State = rsPaused) or (Core.CurrentTarget.State = rsStopped));
  AProgramStepOver.Enabled := (tcStepOver in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned and ((Core.CurrentTarget.State = rsPaused) or (Core.CurrentTarget.State = rsStopped));
  AProgramShowExecutionPoint.Enabled := (tcPause in Core.CurrentTarget.Capabilities) and
    Core.Project.Openned and (Core.CurrentTarget.State = rsPaused);
  AShowSourcePosition.Enabled := Core.CurrentTarget.Compiled;
  AShowTargetPosition.Enabled := Core.CurrentTarget.Compiled;
  UpdateStatusBar;
end;

procedure TFormMain.UpdateStatusBar;
begin
  StatusBarMain.Panels[0].Text := IntToStr(FormSourceCode.MemoSource.CaretPos.X) +
    ', ' + IntToStr(FormSourceCode.MemoSource.CaretPos.Y);
end;

procedure TFormMain.UpdateTargetList;
var
  I: Integer;
begin
  Core.Targets.LoadToMenuItem(MenuItemTarget, MenuItemTargetClick, Core.CurrentTarget);
  Core.Targets.LoadToMenuItem(PopupMenuTargets.Items, MenuItemTargetClick, Core.CurrentTarget);

  // Update toolbutton
  I := Core.Targets.IndexOf(Core.CurrentTarget);
  if I <> -1 then
  with TTarget(Core.Targets[I]) do begin
    ToolButtonTarget.ImageIndex := ImageIndex;
    ToolButtonTarget.Caption := Name;
    ToolButtonTarget.OnClick := MenuItemTargetClick();
  end;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  UpdateTargetList;
  Core.LastOpenedList.OnChange := LastOpenedListChange;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  SaveToRegistry(HKEY(Core.ApplicationInfo.RegistryRoot), Core.ApplicationInfo.RegistryKey);
end;

procedure TFormMain.AProgramRunExecute(Sender: TObject);
begin
  if Core.CurrentTarget is TTargetInterpretter then
    TTargetInterpretter(Core.CurrentTarget).Input := FormInput.MemoInput.Lines.Text;
  if not Core.CurrentTarget.Compiled then AProgramCompile.Execute;
  Core.CurrentTarget.Run;
  Core.CurrentTarget.Messages.AddMessage(SProgramStarted);
end;

procedure TFormMain.AShowSourcePositionExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
begin
  DebugStep := Core.CurrentTarget.DebugSteps.SearchByTargetPos(FormTargetCode.MemoTarget.SelStart);
  if Assigned(DebugStep) then begin
    FormSourceCode.MemoSource.SelStart := DebugStep.SourcePosition;
    PageControlMain.TabIndex := 0;
  end;
end;

procedure TFormMain.AShowTargetPositionExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
begin
  DebugStep := Core.CurrentTarget.DebugSteps.SearchBySourcePos(FormSourceCode.MemoSource.SelStart);
  if Assigned(DebugStep) then begin
    FormTargetCode.MemoTarget.SelStart := DebugStep.TargetPosition;
    PageControlMain.TabIndex := 1;
  end;
end;

procedure TFormMain.AViewCompilersExecute(Sender: TObject);
begin
  FormCompilers.Show;
end;

procedure TFormMain.AViewInterpretterExecute(Sender: TObject);
begin
  FormCPU.Show;
end;

procedure TFormMain.AViewLogExecute(Sender: TObject);
begin
  FormLog.Show;
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Core.PersistentForm1.Save(Self);
end;

procedure TFormMain.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if Core.Project.Modified then begin
    AProjectClose.Execute;
    if Core.Project.Openned then CanClose := False;
  end;
end;

procedure TFormMain.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.AOptionsExecute(Sender: TObject);
begin
  OptionsForm.Load;
  if OptionsForm.ShowModal = mrOK then begin
    OptionsForm.Save;
    if Assigned(Core.CurrentTarget) then begin
      Core.CurrentTarget.OptimizationLevel := Core.OptimizationLevel;
      if Core.CurrentTarget is TBFTarget then begin
        TBFTarget(Core.CurrentTarget).CellSize := Core.CellSize;
        TBFTarget(Core.CurrentTarget).MemorySize := Core.MemorySize;
      end;
    end;
  end;
end;

procedure TFormMain.AProgramCompileExecute(Sender: TObject);
begin
  with Core.CurrentTarget do begin
    AProgramStop.Execute;
    SourceCode := FormSourceCode.MemoSource.Text;
    ProjectFileName := Core.Project.FileName;
    ProgramName := ExtractFileNameOnly(Core.Project.FileName);
    Core.CurrentTarget.Messages.AddMessage(SCompileStart);
    Compile;
    Core.CurrentTarget.Messages.AppendMessage(SDone);
    FormTargetCode.MemoTarget.Text := TargetCode;
    UpdateInterface;
  end;
end;

procedure TFormMain.ABreakpointToggleExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
  BreakPoint: TBreakPoint;
begin
  DebugStep := Core.CurrentTarget.DebugSteps.SearchBySourcePos(FormSourceCode.MemoSource.SelStart);
  if Assigned(DebugStep) then begin
    BreakPoint := Core.CurrentTarget.BreakPoints.SearchByTargetPos(DebugStep.TargetPosition);
    if Assigned(BreakPoint) then
      Core.CurrentTarget.BreakPoints.Delete(Core.CurrentTarget.BreakPoints.IndexOf(BreakPoint))
      else Core.CurrentTarget.BreakPoints.AddItem(DebugStep.TargetPosition);
  end;
end;

procedure TFormMain.AAboutExecute(Sender: TObject);
begin
  FormAbout := TFormAbout.Create(nil);
  try
    FormAbout.ApplicationInfo := Core.ApplicationInfo;
    FormAbout.CoolTranslator := Core.CoolTranslator1;
    FormAbout.ShowModal;
  finally
    FreeAndNil(FormAbout);
  end;
end;

procedure TFormMain.AProgramPauseExecute(Sender: TObject);
begin
  Core.CurrentTarget.Pause;
end;

procedure TFormMain.AProgramRunToCursorExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
begin
  if Core.CurrentTarget.State = rsStopped then begin
    if not Core.CurrentTarget.Compiled then AProgramCompile.Execute;
    DebugStep := Core.CurrentTarget.DebugSteps.SearchBySourcePos(FormSourceCode.MemoSource.SelStart);
    Core.CurrentTarget.BreakPoints.SetSystem(DebugStep.TargetPosition);
    AProgramRun.Execute;
  end else begin
    DebugStep := Core.CurrentTarget.DebugSteps.SearchBySourcePos(FormSourceCode.MemoSource.SelStart);
    Core.CurrentTarget.RunToCursor(DebugStep.TargetPosition);
  end;
end;

procedure TFormMain.AProgramShowExecutionPointExecute(Sender: TObject);
var
  DebugStep: TDebugStep;
begin
  DebugStep := Core.CurrentTarget.DebugSteps.SearchByTargetPos(Core.CurrentTarget.ExecutionPosition);
  FormSourceCode.MemoSource.SelStart := DebugStep.SourcePosition;
  FormTargetCode.MemoTarget.SelStart := DebugStep.TargetPosition;
  if PageControlMain.TabIndex = 0 then FormSourceCode.MemoSource.SetFocus;
  if PageControlMain.TabIndex = 1 then FormTargetCode.MemoTarget.SetFocus;
end;

procedure TFormMain.AProgramStepIntoExecute(Sender: TObject);
begin
  if Core.CurrentTarget.State = rsStopped then begin
    if not Core.CurrentTarget.Compiled then AProgramCompile.Execute;
    Core.CurrentTarget.BreakPoints.SetSystem(TDebugStep(Core.CurrentTarget.DebugSteps.First).TargetPosition);
    AProgramRun.Execute;
  end else Core.CurrentTarget.StepInto;
end;

procedure TFormMain.AProgramStepOutExecute(Sender: TObject);
begin
  Core.CurrentTarget.StepOut;
end;

procedure TFormMain.AProgramStepOverExecute(Sender: TObject);
begin
  if Core.CurrentTarget.State = rsStopped then begin
    if not Core.CurrentTarget.Compiled then AProgramCompile.Execute;
    Core.CurrentTarget.BreakPoints.SetSystem(TDebugStep(Core.CurrentTarget.DebugSteps.First).TargetPosition);
    AProgramRun.Execute;
  end else Core.CurrentTarget.StepOver;
end;

procedure TFormMain.AProgramStopExecute(Sender: TObject);
begin
  Core.CurrentTarget.Stop;
end;

procedure TFormMain.AProjectCloseExecute(Sender: TObject);
var
  DialogResult: TModalResult;
begin
  DialogResult := mrNo;
  if Core.Project.Modified then begin
    DialogResult := MessageDlg(SUnsavedChanges, SDoYouWantToSaveProject, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
    if DialogResult = mrYes then begin
      AProjectSave.Execute;
    end;
  end;
  if (DialogResult = mrNo) or (DialogResult = mrYes) then begin
    AProgramStop.Execute;
    Core.Project.Openned := False;
    FormSourceCode.MemoSource.Clear;
    UpdateInterface;
  end;
end;

procedure TFormMain.AProjectNewExecute(Sender: TObject);
begin
  AProjectClose.Execute;
  if Core.Project.Openned then Exit;
  Core.Project.FileName := 'Examples' + DirectorySeparator + SNewProject + '.b';
  FormSourceCode.MemoSource.Clear;
  Core.Project.Openned := True;
  Core.CurrentTarget.Compiled := False;
  UpdateInterface;
end;

procedure TFormMain.AProjectOpenRecentExecute(Sender: TObject);
begin
  AProjectClose.Execute;
  if Core.Project.Openned then Exit;
  ProjectOpen(Core.LastOpenedList.Items[TMenuItem(Sender).MenuIndex]);
end;

procedure TFormMain.AProjectOpenExecute(Sender: TObject);
begin
  if Core.LastOpenedList.Items.Count > 0 then
    OpenDialog1.FileName := Core.LastOpenedList.Items[0]
    else OpenDialog1.FileName := ExtractFileDir(Application.ExeName) + DirectorySeparator + 'Examples';
  OpenDialog1.InitialDir := ExtractFileDir(OpenDialog1.FileName);
  if OpenDialog1.Execute then begin
    AProjectClose.Execute;
    if Core.Project.Openned then Exit;
    ProjectOpen(OpenDialog1.FileName);
  end;
end;

procedure TFormMain.AProjectSaveAsExecute(Sender: TObject);
begin
  SaveDialog1.FileName := Core.Project.FileName;
  if SaveDialog1.Execute then begin
    FormSourceCode.MemoSource.Lines.SaveToFile(SaveDialog1.FileName);
    Core.Project.FileName := SaveDialog1.FileName;
    Core.LastOpenedList.AddItem(SaveDialog1.FileName);
    Core.Project.Modified := False;
    UpdateInterface;
  end;
end;

procedure TFormMain.AProjectSaveExecute(Sender: TObject);
begin
  if Core.Project.Modified and FileExistsUTF8(Core.Project.FileName) then begin
    FormSourceCode.MemoSource.Lines.SaveToFile(Core.Project.FileName);
    Core.Project.Modified := False;
    UpdateInterface;
  end else AProjectSaveAs.Execute;
end;

end.

