unit UFormSourceCode;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Menus, ActnList, strutils;

type

  { TFormSourceCode }

  TFormSourceCode = class(TForm)
    ACutToClipboard: TAction;
    ASelectAll: TAction;
    APasteFromClipboard: TAction;
    ACopyToClipboard: TAction;
    AClear: TAction;
    AFormatSource: TAction;
    AShrinkSource: TAction;
    AGenerateNumber: TAction;
    ActionList1: TActionList;
    MemoSource: TMemo;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem21: TMenuItem;
    MenuItem23: TMenuItem;
    MenuItem26: TMenuItem;
    MenuItem29: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem36: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    PopupMenuSource: TPopupMenu;
    procedure AClearExecute(Sender: TObject);
    procedure ACopyToClipboardExecute(Sender: TObject);
    procedure ACutToClipboardExecute(Sender: TObject);
    procedure AFormatSourceExecute(Sender: TObject);
    procedure AGenerateNumberExecute(Sender: TObject);
    procedure APasteFromClipboardExecute(Sender: TObject);
    procedure ASelectAllExecute(Sender: TObject);
    procedure AShrinkSourceExecute(Sender: TObject);
    procedure MemoSourceChange(Sender: TObject);
    procedure MemoSourceKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MemoSourceKeyPress(Sender: TObject; var Key: char);
    procedure MemoSourceKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState
      );
    procedure MemoSourceMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
  public
    procedure UpdateInterface;
  end;

var
  FormSourceCode: TFormSourceCode;


implementation

uses
  UCore, UFormMain;

resourcestring
  SNumberGeneration = 'Number generation';
  SEnterNumber = 'Enter number';


{$R *.lfm}

{ TFormSourceCode }

procedure TFormSourceCode.AGenerateNumberExecute(Sender: TObject);
var
  NumberText: string;
  Number: Integer;
  I: Integer;
begin
  NumberText := InputBox(SNumberGeneration, SEnterNumber, '1');
  if TryStrToInt(NumberText, Number) then begin
    NumberText := '';
    for I := 0 to Number - 1 do
      NumberText := NumberText + '+';
    MemoSource.SelText := NumberText;
  end;
end;

procedure TFormSourceCode.APasteFromClipboardExecute(Sender: TObject);
begin
  MemoSource.PasteFromClipboard;
end;

procedure TFormSourceCode.ASelectAllExecute(Sender: TObject);
begin
  MemoSource.SelectAll;
end;

procedure TFormSourceCode.AFormatSourceExecute(Sender: TObject);
var
  Source: string;
  NewSource: string;
  Indent: Integer;
  I: Integer;
const
  IndentText = '  ';
begin
  Source := MemoSource.Text;
  NewSource := '';
  Indent := 0;
  for I := 1 to Length(Source) do begin
    if Source[I] = '[' then begin
      NewSource := NewSource + LineEnding + DupeString(IndentText, Indent) + Source[I] + LineEnding ;
      Inc(Indent);
      NewSource := NewSource + DupeString(IndentText, Indent);
    end
    else if Source[I] = ']' then begin
      Dec(Indent);
      NewSource := NewSource + LineEnding + DupeString(IndentText, Indent) + Source[I] + LineEnding + DupeString(IndentText, Indent);
    end
    else if Ord(Source[I]) > $20 then
      NewSource := NewSource + Source[I];
  end;
  MemoSource.Text := NewSource;
end;

procedure TFormSourceCode.ACopyToClipboardExecute(Sender: TObject);
begin
  MemoSource.CopyToClipboard;
end;

procedure TFormSourceCode.ACutToClipboardExecute(Sender: TObject);
begin
  MemoSource.CutToClipboard;
end;

procedure TFormSourceCode.AClearExecute(Sender: TObject);
begin
  MemoSource.Clear;
end;

procedure TFormSourceCode.AShrinkSourceExecute(Sender: TObject);
var
  Source: string;
  Pos: Integer;
  I: Integer;
begin
  Source := MemoSource.Text;
  Pos := 1;
  for I := 1 to Length(Source) do begin
    if Source[I] > ' ' then begin
      Source[Pos] := Source[I];
      Inc(Pos);
    end;
  end;
  SetLength(Source, Pos - 1);
  MemoSource.Text := Source;
end;

procedure TFormSourceCode.UpdateInterface;
begin

end;

procedure TFormSourceCode.MemoSourceChange(Sender: TObject);
begin
  Core.Project.Modified := True;
  Core.CurrentTarget.Compiled := False;
  FormMain.UpdateInterface;
end;

procedure TFormSourceCode.MemoSourceKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
end;

procedure TFormSourceCode.MemoSourceKeyPress(Sender: TObject; var Key: char);
begin
end;

procedure TFormSourceCode.MemoSourceKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  FormMain.UpdateStatusBar;
end;

procedure TFormSourceCode.MemoSourceMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FormMain.UpdateStatusBar;
end;


end.

