unit UTargetFPC;

{$mode Delphi}{$H+}

interface

uses
  Classes, SysUtils, UTarget, UBFTarget;

type

  { TTargetFPC }

  TTargetFPC = class(TBFTarget)
  private
  public
    constructor Create; override;
    procedure Compile; override;
    procedure Run; override;
  end;

implementation

{ TTargetFPC }

constructor TTargetFPC.Create;
begin
  inherited Create;
  Name := 'FPC';
  SourceExtension := '.pas';
  ImageIndex := 22;
  Capabilities := [tcCompile, tcRun];
  {$IFDEF Linux}
  CompiledExtension := '';
  CompilerPath := '/usr/bin/fpc';
  RunExtension := '';
  {$ENDIF}
  {$IFDEF Windows}
  CompiledExtension := '.exe';
  CompilerPath := 'fpc.exe';
  RunExtension := '';
  {$ENDIF}
end;

procedure TTargetFPC.Compile;
begin
  inherited;
  Indent := 0;
  FTargetCode := '';

  AddLine('program ' + ProgramName + ';');
  AddLine('');
  AddLine('var');
  AddLine('  Memory: array[0..30000] of Byte;');
  AddLine('  Pos: Integer;');
  AddLine('  ReadChar: Char;');
  AddLine('begin');
  Inc(Indent);
  AddLine('Pos := 0;');
  FProgramIndex := 0;
  while (FProgramIndex < Length(FProgram)) do begin
    case FProgram[FProgramIndex].Command of
      cmPointerInc: AddLine('Inc(Pos, ' + IntToStr(FProgram[FProgramIndex].Parameter) + ');');
      cmPointerDec: AddLine('Dec(Pos, ' + IntToStr(FProgram[FProgramIndex].Parameter) + ');');
      cmInc: AddLine('Memory[Pos] := Memory[Pos] + ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmDec: AddLine('Memory[Pos] := Memory[Pos] - ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmSet: AddLine('Memory[Pos] := ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmOutput: AddLine('Write(Chr(Memory[Pos]));');
      cmInput: AddLine('Read(ReadChar); Memory[Pos] := Ord(ReadChar);');
      cmLoopStart: begin
        AddLine('while Memory[Pos] <> 0 do begin');
        Inc(Indent);
      end;
      cmLoopEnd: begin
        Dec(Indent);
        AddLine('end;');
      end;
    end;
    Inc(FProgramIndex);
  end;
  Dec(Indent);
  AddLine('end.');

  CompileToFile;
end;

procedure TTargetFPC.Run;
begin
  inherited Run;
  RunFromFile;
end;

end.

